@@ -1,12 +1,9 @@
-import datetime
 from abc import abstractmethod, ABC
 from typing import Mapping, Union, List
 
 import pandas as pd
 
-from utils.graphs_util import create_graph_as_dict
-from utils.window_utils import generate_time_windows, generate_session_windows
-from utils.graphs_util import GraphCreator, GraphFromTimeWindowCreator, GraphFromSessionWindowCreator, \
+from utils.graphs_util import GraphFromSessionWindowCreator, \
     GraphFromTumblingWindowCreator, GraphFromSlidingWindowCreator
 
 
@@ -56,10 +53,11 @@ class Dataset(ABC):
         self.load_event_templates()
         self.assign_event_id_to_logs()
 
-    def create_graphs(self, window_type, window_size, window_slide, test_id, include_last=True) \
-            -> List[Mapping[str, Union[datetime.datetime, Mapping[str, Mapping[str, int]]]]]:
-        # TODO add support for session windows as well
+    def create_graphs(self, window_type, window_size, window_slide, test_id, include_last=True,
+                      include_all_event_ids=False) \
+            -> List[Mapping[str, Union[str, Mapping[str, Mapping[str, float]]]]]:
         """
+        :param include_all_event_ids: A bool value that represents whether all detected event IDs should be included in the graph
         :param window_type: string that represents the window type, possible values are: session, tumbling, sliding
         :param window_size: the size of the window in milliseconds
         :param window_slide: the size of the slide of the window in milliseconds
@@ -77,19 +75,25 @@ class Dataset(ABC):
         if window_type == 'tumbling':
             graph_creator = GraphFromTumblingWindowCreator(
                 logs=logs,
+                templates=self.templates,
                 include_last=include_last,
+                include_all_event_ids=include_all_event_ids,
                 window_size=window_size)
         elif window_type == 'sliding':
             graph_creator = GraphFromSlidingWindowCreator(
                 logs=logs,
+                templates=self.templates,
                 include_last=include_last,
+                include_all_event_ids=include_all_event_ids,
                 window_size=window_size,
                 window_slide=window_slide
             )
         else:
             graph_creator = GraphFromSessionWindowCreator(
                 logs=logs,
-                include_last=include_last
+                templates=self.templates,
+                include_last=include_last,
+                include_all_event_ids=include_all_event_ids
             )
 
-        return graph_creator.create_graphs()
\ No newline at end of file
+        return graph_creator.create_graphs()
