@@ -118,6 +118,7 @@ class BGLDataset(Dataset):
         parser.parse(log_file)
 
         self.logs = pd.read_csv(output_dir + 'processed_logs.log_structured.csv')
+        self.templates=pd.read_csv(output_dir + 'processed_logs.log_templates.csv')
         self.logs['Time'] = pd.to_datetime(self.logs['Time'], format='%Y-%m-%d-%H.%M.%S.%f')
         self.logs.sort_values(['Time'])
 
@@ -166,6 +167,7 @@ class NovaDataset(Dataset):
         url = 'https://drive.google.com/uc?id=1SZqgSvUhvwZTofKslo21U3z2VEBbU17w'
         if not exists(output_path):
             gdown.download(url, output_path, quiet=False)
+        self.templates = pd.read_csv(f'{self.data_folder_path}/nova/event_templates.csv')
 
     def assign_event_id_to_logs(self):
         """
