@@ -1,31 +1,37 @@
 from abc import abstractmethod, ABC
+from typing import List, Mapping, Union
 
 import networkx as nx
 import pandas as pd
 
-from utils.window_utils import TumblingWindowCreator, SlidingWindowCreator
+import log_datasets.datasets
+from utils.window_utils import TumblingWindowCreator, SlidingWindowCreator, WindowsCreator
+# from log_datasets.datasets import NovaDataset
 
 
 class GraphCreator(ABC):
-    def __init__(self, logs: pd.DataFrame, include_last: bool):
+    def __init__(self, logs: pd.DataFrame, templates: pd.DataFrame, include_last: bool, include_all_event_ids: bool):
         self.logs = logs
+        self.templates = templates
         self.include_last = include_last
+        self.include_all_event_ids = include_all_event_ids
 
     @abstractmethod
-    def create_graphs(self):
+    def create_graphs(self) -> List[Mapping[str, Union[str, Mapping[str, Mapping[str, float]]]]]:
         pass
 
     @abstractmethod
-    def get_window_creator(self):
+    def get_window_creator(self) -> WindowsCreator:
         pass
 
 
 class GraphFromTimeWindowCreator(GraphCreator, ABC):
-    def __init__(self, logs: pd.DataFrame, include_last: bool, window_size: int):
-        GraphCreator.__init__(self, logs, include_last)
+    def __init__(self, logs: pd.DataFrame, templates: pd.DataFrame, include_last: bool, include_all_event_ids: bool,
+                 window_size: int):
+        GraphCreator.__init__(self, logs, templates, include_last, include_all_event_ids)
         self.window_size = window_size
 
-    def create_graphs(self):
+    def create_graphs(self) -> List[Mapping[str, Union[str, Mapping[str, Mapping[str, float]]]]]:
         windows = self.get_window_creator().create_windows()
         windows['event_ids'] = windows.apply(lambda window: get_events_ids_for_window(self.logs, window), axis=1)
         result = []
@@ -33,19 +39,23 @@ class GraphFromTimeWindowCreator(GraphCreator, ABC):
             # Hack fix, think of a better one later
             r = [] if pd.isnull(row['event_ids']) else row['event_ids'].split(',')
             result.append({
-                'start': row['start'],
-                'end': row['end'],
-                'graph_dict': create_graph_as_dict(event_ids=r, include_last=self.include_last)
+                'start': str(row['start']),
+                'end': str(row['end']),
+                'graph_dict': create_graph_as_dict(
+                    event_ids=r,
+                    include_last=self.include_last,
+                    all_event_ids=list(self.templates['EventId']) if self.include_all_event_ids else [])
             })
 
         return result
 
 
 class GraphFromTumblingWindowCreator(GraphFromTimeWindowCreator):
-    def __init__(self, logs: pd.DataFrame, include_last: bool, window_size: int):
-        GraphFromTimeWindowCreator.__init__(self, logs, include_last, window_size)
+    def __init__(self, logs: pd.DataFrame, templates: pd.DataFrame, include_last: bool, include_all_event_ids: bool,
+                 window_size: int):
+        GraphFromTimeWindowCreator.__init__(self, logs, templates, include_last, include_all_event_ids, window_size)
 
-    def get_window_creator(self):
+    def get_window_creator(self) -> WindowsCreator:
         return TumblingWindowCreator(
             min_timestamp=self.logs['timestamp'].min(),
             max_timestamp=self.logs['timestamp'].max(),
@@ -54,11 +64,12 @@ class GraphFromTumblingWindowCreator(GraphFromTimeWindowCreator):
 
 
 class GraphFromSlidingWindowCreator(GraphFromTimeWindowCreator):
-    def __init__(self, logs: pd.DataFrame, include_last: bool, window_size: int, window_slide: int):
-        GraphFromTimeWindowCreator.__init__(self, logs, include_last, window_size)
+    def __init__(self, logs: pd.DataFrame, templates: pd.DataFrame, include_last: bool, include_all_event_ids: bool,
+                 window_size: int, window_slide: int):
+        GraphFromTimeWindowCreator.__init__(self, logs, templates, include_last, include_all_event_ids, window_size)
         self.window_slide = window_slide
 
-    def get_window_creator(self):
+    def get_window_creator(self) -> WindowsCreator:
         return SlidingWindowCreator(
             min_timestamp=self.logs['timestamp'].min(),
             max_timestamp=self.logs['timestamp'].max(),
@@ -68,10 +79,10 @@ class GraphFromSlidingWindowCreator(GraphFromTimeWindowCreator):
 
 
 class GraphFromSessionWindowCreator(GraphCreator):
-    def __init__(self, logs: pd.DataFrame, include_last: bool):
-        GraphCreator.__init__(self, logs, include_last)
+    def __init__(self, logs: pd.DataFrame, templates: pd.DataFrame, include_last: bool, include_all_event_ids: bool):
+        GraphCreator.__init__(self, logs, templates, include_last, include_all_event_ids)
 
-    def create_graphs(self):
+    def create_graphs(self) -> List[Mapping[str, Union[str, Mapping[str, Mapping[str, float]]]]]:
         windows_df = self.logs[['session_id', 'event_id']]
         windows_df['event_ids'] = windows_df.groupby('session_id')['event_id'].transform(lambda x: ','.join(x))
         windows_df = windows_df[['session_id', 'event_ids']].drop_duplicates()
@@ -81,13 +92,16 @@ class GraphFromSessionWindowCreator(GraphCreator):
             # Hack fix, think of a better one later
             r = [] if pd.isnull(row['event_ids']) else row['event_ids'].split(',')
             result.append({
-                'session_id': row['end'],
-                'graph_dict': create_graph_as_dict(event_ids=r, include_last=self.include_last)
+                'session_id': row['session_id'],
+                'graph_dict': create_graph_as_dict(
+                    event_ids=r,
+                    include_last=self.include_last,
+                    all_event_ids=list(self.templates['EventId']) if self.include_all_event_ids else [])
             })
 
         return result
 
-    def get_window_creator(self):
+    def get_window_creator(self) -> WindowsCreator:
         pass
 
 
@@ -98,9 +112,10 @@ def get_events_ids_for_window(logs_df, window):
                              ]['event_id']))
 
 
-def create_graph_as_dict(event_ids: list, include_last: bool) -> dict:
+def create_graph_as_dict(event_ids: list, include_last: bool, all_event_ids: list()) -> dict:
     """
     Method that creates a graph represented as a dictionary based on the occurrences of event ids in a window/session
+    :param all_event_ids:
     :param include_last: A bool value that represents whether the last event ID should be included in the graph generation
     :param event_ids: List of the event ids extracted from the logs that occurred in a given window/session
     :return: A dictionary for representation of the graph (nodes and edges),
@@ -122,6 +137,10 @@ def create_graph_as_dict(event_ids: list, include_last: bool) -> dict:
         else:
             graph_dict[node1] = dict()
             graph_dict[node1][node2] = 1
+    # Add the template event IDs that were not found in window in the graph as nodes without connections
+    for event_id in all_event_ids:
+        if event_id not in event_ids:
+            graph_dict[event_id]={}
     return normalize_weights(graph_dict)
 
 
@@ -204,5 +223,17 @@ def normalize_weights(graph: dict) -> dict:
 
 
 if __name__ == '__main__':
-    print(create_graph_as_dict("A,B,C,B,C,A,D".split(","), False))
-    print(create_position_embeddings("A,B,C,B,C,A,D".split(","), False))
+    nova_dataset = log_datasets.datasets.NovaDataset("../../data")
+    nova_dataset.initialize_dataset()
+    graph_dicts = nova_dataset.create_graphs(
+        window_type="sliding",
+        window_size=60000,
+        window_slide=2000,
+        include_last=True,
+        include_all_event_ids=True,
+        test_id=1
+    )
+    for graph_dict in graph_dicts:
+        print (graph_dict)
+    # print(create_graph_as_dict("A,B,C,B,C,A,D".split(","), False, list()))
+    # print(create_position_embeddings("A,B,C,B,C,A,D".split(","), False))
